@@ -3,13 +3,33 @@
 """
 Defines a request dispatcher, a HTTP request handler, a HTTP server and a
 CGI request handler.
+
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+   http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+
+:license: Apache License 2.0
+:version: 0.1.6
 """
 
-__version__ = "0.1.5"
+# Module version
+__version_info__ = (0, 1, 6)
+__version__ = ".".join(str(x) for x in __version_info__)
+
+# Documentation strings format
+__docformat__ = "restructuredtext en"
 
 # ------------------------------------------------------------------------------
 # Local modules
-from jsonrpclib import Fault, config
+from jsonrpclib import Fault
 import jsonrpclib
 import jsonrpclib.utils as utils
 
@@ -19,29 +39,29 @@ import sys
 import traceback
 
 try:
-    import fcntl
+    # Python 3
+    import xmlrpc.server as xmlrpcserver
+    import socketserver
+
 except ImportError:
-    # For Windows
-    fcntl = None
-
-# ------------------------------------------------------------------------------
-
-if sys.version_info[0] < 3:
     # Python 2
     import SimpleXMLRPCServer as xmlrpcserver
     import SocketServer as socketserver
 
-else:
-    # Python 3
-    import xmlrpc.server as xmlrpcserver
-    import socketserver
+try:
+    # Windows
+    import fcntl
+
+except ImportError:
+    # Others
+    fcntl = None
 
 # ------------------------------------------------------------------------------
 
 def get_version(request):
     """
     Computes the JSON-RPC version
-    
+
     :param request: A request dictionary
     :return: The JSON-RPC version or None
     """
@@ -54,17 +74,19 @@ def get_version(request):
     return None
 
 
-def validate_request(request):
+def validate_request(request, json_config):
     """
     Validates the format of a request dictionary
-    
+
     :param request: A request dictionary
+    :param json_config: A JSONRPClib Config instance
     :return: True if the dictionary is valid, else a Fault object
     """
     if not isinstance(request, utils.DictType):
         # Invalid request type
         return Fault(-32600, 'Request must be a dict, not {0}' \
-                     .format(type(request).__name__))
+                     .format(type(request).__name__),
+                     config=json_config)
 
     # Get the request ID
     rpcid = request.get('id', None)
@@ -73,7 +95,7 @@ def validate_request(request):
     version = get_version(request)
     if not version:
         return Fault(-32600, 'Request {0} invalid.'.format(request),
-                     rpcid=rpcid)
+                     rpcid=rpcid, config=json_config)
 
     # Default parameters: empty list
     request.setdefault('params', [])
@@ -87,7 +109,7 @@ def validate_request(request):
     not isinstance(params, param_types):
         # Invalid type of method name or parameters
         return Fault(-32600, 'Invalid request parameters or method.',
-                     rpcid=rpcid)
+                     rpcid=rpcid, config=json_config)
 
     # Valid request
     return True
@@ -103,7 +125,7 @@ class NoMulticallResult(Exception):
 
 class SimpleJSONRPCDispatcher(xmlrpcserver.SimpleXMLRPCDispatcher):
 
-    def __init__(self, encoding=None):
+    def __init__(self, encoding=None, config=jsonrpclib.config.DEFAULT):
         """
         Sets up the dispatcher with the given encoding.
         None values are allowed.
@@ -112,16 +134,18 @@ class SimpleJSONRPCDispatcher(xmlrpcserver.SimpleXMLRPCDispatcher):
             # Default encoding
             encoding = "UTF-8"
 
+
         xmlrpcserver.SimpleXMLRPCDispatcher.__init__(self,
                                                      allow_none=True,
                                                      encoding=encoding)
+        self.json_config = config
 
 
     def _unmarshaled_dispatch(self, request, dispatch_method=None):
         """
         Loads the request dictionary (unmarshaled), calls the method(s)
         accordingly and returns a JSON-RPC dictionary (not marshaled)
-        
+
         :param request: JSON-RPC request dictionary (or list of)
         :param dispatch_method: Custom dispatch method (for method resolution)
         :return: A JSON-RPC dictionary (or an array of) or None if the request
@@ -130,7 +154,8 @@ class SimpleJSONRPCDispatcher(xmlrpcserver.SimpleXMLRPCDispatcher):
         """
         if not request:
             # Invalid request dictionary
-            fault = Fault(-32600, 'Request invalid -- no request data.')
+            fault = Fault(-32600, 'Request invalid -- no request data.',
+                          config=self.json_config)
             return fault.dump()
 
         if type(request) is utils.ListType:
@@ -138,7 +163,7 @@ class SimpleJSONRPCDispatcher(xmlrpcserver.SimpleXMLRPCDispatcher):
             responses = []
             for req_entry in request:
                 # Validate the request
-                result = validate_request(req_entry)
+                result = validate_request(req_entry, self.json_config)
                 if type(result) is Fault:
                     responses.append(result.dump())
                     continue
@@ -162,7 +187,7 @@ class SimpleJSONRPCDispatcher(xmlrpcserver.SimpleXMLRPCDispatcher):
 
         else:
             # Single call
-            result = validate_request(request)
+            result = validate_request(request, self.json_config)
             if type(result) is Fault:
                 return result.dump()
 
@@ -179,19 +204,20 @@ class SimpleJSONRPCDispatcher(xmlrpcserver.SimpleXMLRPCDispatcher):
         """
         Parses the request data (marshaled), calls method(s) and returns a
         JSON string (marshaled)
-        
+
         :param data: A JSON request string
         :param dispatch_method: Custom dispatch method (for method resolution)
         :return: A JSON-RPC response string (marshaled)
         """
         # Parse the request
         try:
-            request = jsonrpclib.loads(data)
+            request = jsonrpclib.loads(data, self.json_config)
 
         except Exception as ex:
             # Parsing/loading error
             fault = Fault(-32700, 'Request {0} invalid. ({1}:{2})' \
-                          .format(data, type(ex).__name__, ex))
+                          .format(data, type(ex).__name__, ex),
+                          config=self.json_config)
             return fault.response()
 
         # Get the response dictionary
@@ -214,7 +240,7 @@ class SimpleJSONRPCDispatcher(xmlrpcserver.SimpleXMLRPCDispatcher):
     def _marshaled_single_dispatch(self, request, dispatch_method=None):
         """
         Dispatches a single method call
-        
+
         :param request: A validated request dictionary
         :param dispatch_method: Custom dispatch method (for method resolution)
         :return: A JSON-RPC response dictionary, or None if it was a
@@ -234,7 +260,8 @@ class SimpleJSONRPCDispatcher(xmlrpcserver.SimpleXMLRPCDispatcher):
         except:
             # Return a fault
             exc_type, exc_value, _ = sys.exc_info()
-            fault = Fault(-32603, '{0}:{1}'.format(exc_type, exc_value))
+            fault = Fault(-32603, '{0}:{1}'.format(exc_type, exc_value),
+                          config=self.json_config)
             return fault.dump()
 
         if 'id' not in request or request['id'] in (None, ''):
@@ -245,19 +272,20 @@ class SimpleJSONRPCDispatcher(xmlrpcserver.SimpleXMLRPCDispatcher):
         # Prepare a JSON-RPC dictionary
         try:
             return jsonrpclib.dump(response, rpcid=request['id'],
-                                   is_response=True)
+                                   is_response=True, config=self.json_config)
 
         except:
             # JSON conversion exception
             exc_type, exc_value, _ = sys.exc_info()
-            fault = Fault(-32603, '{0}:{1}'.format(exc_type, exc_value))
+            fault = Fault(-32603, '{0}:{1}'.format(exc_type, exc_value),
+                          config=self.json_config)
             return fault.dump()
 
 
     def _dispatch(self, method, params):
         """
         Default method resolver and caller
-        
+
         :param method: Name of the method to call
         :param params: List of arguments to give to the method
         :return: The result of the method
@@ -294,23 +322,29 @@ class SimpleJSONRPCDispatcher(xmlrpcserver.SimpleXMLRPCDispatcher):
 
             except TypeError as ex:
                 # Maybe the parameters are wrong
-                return Fault(-32602, 'Invalid parameters: {0}'.format(ex))
+                return Fault(-32602, 'Invalid parameters: {0}'.format(ex),
+                             config=self.json_config)
 
             except:
                 # Method exception
                 err_lines = traceback.format_exc().splitlines()
                 trace_string = '{0} | {1}'.format(err_lines[-3], err_lines[-1])
-                return Fault(-32603, 'Server error: {0}'.format(trace_string))
+                return Fault(-32603, 'Server error: {0}'.format(trace_string),
+                             config=self.json_config)
 
         else:
             # Unknown method
-            return Fault(-32601, 'Method {0} not supported.'.format(method))
+            return Fault(-32601, 'Method {0} not supported.'.format(method),
+                         config=self.json_config)
 
 # ------------------------------------------------------------------------------
 
 class SimpleJSONRPCRequestHandler(xmlrpcserver.SimpleXMLRPCRequestHandler):
     """
-    HTTP server request handler
+    HTTP request handler.
+
+    The server that receives the requests must have a json_config member,
+    containing a JSONRPClib Config instance
     """
     def do_POST(self):
         """
@@ -320,6 +354,9 @@ class SimpleJSONRPCRequestHandler(xmlrpcserver.SimpleXMLRPCRequestHandler):
             self.report_404()
             return
 
+        # Retrieve the configuration
+        config = getattr(self.server, 'json_config', jsonrpclib.config.DEFAULT)
+
         try:
             # Read the request body
             max_chunk_size = 10 * 1024 * 1024
@@ -343,7 +380,7 @@ class SimpleJSONRPCRequestHandler(xmlrpcserver.SimpleXMLRPCRequestHandler):
             err_lines = traceback.format_exc().splitlines()
             trace_string = '{0} | {1}'.format(err_lines[-3], err_lines[-1])
             fault = jsonrpclib.Fault(-32603, 'Server error: {0}'\
-                                     .format(trace_string))
+                                     .format(trace_string), config=config)
             response = fault.response()
 
         if response is None:
@@ -372,23 +409,35 @@ class SimpleJSONRPCServer(socketserver.TCPServer, SimpleJSONRPCDispatcher):
 
     def __init__(self, addr, requestHandler=SimpleJSONRPCRequestHandler,
                  logRequests=True, encoding=None, bind_and_activate=True,
-                 address_family=socket.AF_INET):
+                 address_family=socket.AF_INET,
+                 config=jsonrpclib.config.DEFAULT):
         """
         Sets up the server and the dispatcher
-        
+
         :param addr: The server listening address
         :param requestHandler: Custom request handler
         :param logRequests: Flag to(de)activate requests logging
         :param encoding: The dispatcher request encoding
         :param bind_and_activate: If True, starts the server immediately
         :param address_family: The server listening address family
+        :param config: A JSONRPClib Config instance
         """
         # Set up the dispatcher fields
-        SimpleJSONRPCDispatcher.__init__(self, encoding)
+        SimpleJSONRPCDispatcher.__init__(self, encoding, config)
 
         # Prepare the server configuration
         self.logRequests = logRequests
         self.address_family = address_family
+        self.json_config = config
+
+        # Work on the request handler
+        class RequestHandlerWrapper(requestHandler):
+            def __init__(self, *args, **kwargs):
+                """
+                Constructs the wrapper after having stored the configuration
+                """
+                self.config = config
+                super(RequestHandlerWrapper, self).__init__(*args, **kwargs)
 
         # Set up the server
         socketserver.TCPServer.__init__(self, addr, requestHandler,
@@ -406,20 +455,23 @@ class CGIJSONRPCRequestHandler(SimpleJSONRPCDispatcher):
     """
     JSON-RPC CGI handler (and dispatcher)
     """
-    def __init__(self, encoding=None):
+    def __init__(self, encoding=None, config=jsonrpclib.config.DEFAULT):
         """
         Sets up the dispatcher
-        
+
         :param encoding: Dispatcher encoding
+        :param config: A JSONRPClib Config instance
         """
-        SimpleJSONRPCDispatcher.__init__(self, encoding)
+        SimpleJSONRPCDispatcher.__init__(self, encoding, config)
+
 
     def handle_jsonrpc(self, request_text):
         """
         Handle a JSON-RPC request
         """
         response = self._marshaled_dispatch(request_text)
-        sys.stdout.write('Content-Type: {0}\r\n'.format(config.content_type))
+        sys.stdout.write('Content-Type: {0}\r\n' \
+                         .format(self.json_config.content_type))
         sys.stdout.write('Content-Length: {0:d}\r\n'.format(len(response)))
         sys.stdout.write('\r\n')
         sys.stdout.write(response)
