@@ -48,11 +48,11 @@ appropriately.
 See https://github.com/tcalmant/jsonrpclib for more info.
 
 :license: Apache License 2.0
-:version: 0.1.6
+:version: 0.1.6.1
 """
 
 # Module version
-__version_info__ = (0, 1, 6)
+__version_info__ = (0, 1, 6, 1)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -386,6 +386,34 @@ class ServerProxy(XMLServerProxy):
         # Same as original, just with new _Method reference
         return _Method(self._request, name)
 
+    def __close(self):
+        """
+        Closes the transport layer
+        """
+        try:
+            self.__transport.close()
+
+        except AttributeError:
+            # Not available in Python 2.6
+            pass
+
+
+    def __call__(self, attr):
+        """
+        A workaround to get special attributes on the ServerProxy
+        without interfering with the magic __getattr__
+
+        (code from xmlrpclib in Python 2.7)
+        """
+        if attr == "close":
+            return self.__close
+
+        elif attr == "transport":
+            return self.__transport
+
+        raise AttributeError("Attribute {0} not found".format(attr))
+
+
     @property
     def _notify(self):
         # Just like __getattr__, but with notify namespace.
