@@ -3,11 +3,29 @@
 """
 Utility methods, for compatibility between Python version
 
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+   http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+
 :author: Thomas Calmant
-:version: 1.0.0
+:license: Apache License 2.0
+:version: 1.0.1
 """
 
-__version__ = "1.0.0"
+# Module version
+__version_info__ = (1, 0, 1)
+__version__ = ".".join(str(x) for x in __version_info__)
+
+# Documentation strings format
+__docformat__ = "restructuredtext en"
 
 # ------------------------------------------------------------------------------
 
@@ -19,15 +37,6 @@ if sys.version_info[0] < 3:
     # Python 2
     import types
     StringTypes = types.StringTypes
-    TupleType = types.TupleType
-    ListType = types.ListType
-    DictType = types.DictType
-
-    iter_types = (
-        types.DictType,
-        types.ListType,
-        types.TupleType
-    )
 
     string_types = (
         types.StringType,
@@ -40,11 +49,6 @@ if sys.version_info[0] < 3:
         types.FloatType
     )
 
-    value_types = (
-        types.BooleanType,
-        types.NoneType
-    )
-
     def to_bytes(string):
         """
         Converts the given string into bytes
@@ -68,15 +72,6 @@ if sys.version_info[0] < 3:
 else:
     # Python 3
     StringTypes = (str,)
-    TupleType = tuple
-    ListType = list
-    DictType = dict
-
-    iter_types = (
-        dict,
-        list,
-        tuple
-    )
 
     string_types = (
         bytes,
@@ -88,11 +83,6 @@ else:
         float
     )
 
-    value_types = (
-        bool,
-        type(None)
-    )
-
     def to_bytes(string):
         """
         Converts the given string into bytes
@@ -113,4 +103,22 @@ else:
 
 # ------------------------------------------------------------------------------
 # Common
+
+DictType = dict
+
+ListType = list
+SetTypes = (set, frozenset)
+TupleType = tuple
+
+iterable_types = (
+    list,
+    set, frozenset,
+    tuple
+)
+
+value_types = (
+    bool,
+    type(None)
+)
+
 primitive_types = string_types + numeric_types + value_types
